# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from django.core.exceptions import ValidationError
from project.view import FuncView
from project.forms.fields.extras.enable import EnableField
from project.forms.fields.extras.fqdn import FQDNField
from project.forms.fields.extras.landiskpass import LanDiskPassField
from project.fal.features import Features
from project.fal.clone import Clone


class View(FuncView):

    class Form(forms.Form):
        enable = EnableField(label=_(u'LanDiskClone: I am Master.'))
        slave = FQDNField(label=_(u'LanDiskClone: Slave'), required=False)
        passwd = LanDiskPassField(label=_(u'Password'), required=False)

        def clean(self):
            data = self.cleaned_data
            if self.cleaned_data.get('enable', None):
                if self.cleaned_data.get('slave', None) in [""]:
                    raise ValidationError("%s: %s" % (
                            _(u'LanDiskClone: Slave'),
                            _(u'This field is required.')))
                # NOTE LAN DISKのパスワードは空OKだがレプリケーションではNG
                if self.cleaned_data.get('passwd', None) in [""]:
                    raise ValidationError("%s: %s" % (
                            _(u'Password'),
                            _(u'This field is required.')))
            return data

    views = ['apply']

    def __is_available_replication(self):
        FUNCTION_REPLICATION = "replication"
        OLDEST_VERSION = 1

        features = Features()
        try:
            config_xml = features.query(to_dict=False).get()
        except:
            return False

        for function in config_xml.get_node_list('functionlist/function'):
            version = float(function.get_attr_value('version', "0"))
            if (function.get_text() == FUNCTION_REPLICATION and
                    version >= OLDEST_VERSION):

                return True

        return False

    def view(self, request):
        clone = Clone()
        config = clone.query()
        form = self.Form(initial={
            'enable': config['master']['enable'],
            'slave': config['master']['slave']['host'],
            'passwd': config['master']['slave']['passwd'],
        })
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form,
                'replication_available': self.__is_available_replication(),
            },
        }

    def view_apply(self, request):
        with self.processing() as p:
            form = self.Form(data=request.POST)
            p.break_if_state_change()

            if form.is_valid():
                clone = Clone()
                config = {
                    'master': {
                        'enable': form.cleaned_data['enable'],
                        'slave': {
                            'host': form.cleaned_data['slave'],
                            'passwd': form.cleaned_data['passwd'],
                        },
                    }
                }
                p.result(clone.apply(config=config))
                self.set_message(clone.get_message())
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form},
        }
